package budget

import (
	"context"
	"fmt"

	"git.3crabs.ru/save_my_money/smm_core/internal/services/category"
	"git.3crabs.ru/save_my_money/smm_core/internal/services/context_utils"
	"github.com/jackc/pgx/v5"
	"github.com/jackc/pgx/v5/pgxpool"
)

type BudgetEntity struct {
	Id           int
	Name         string
	StartDay     int
	MonthlyLimit int
	Categories   []*category.CategoryEntity
}

type BudgetService struct {
	db              *pgxpool.Pool
	categoryService *category.CategoryService
}

func NewBudgetService(
	db *pgxpool.Pool,
	categoryService *category.CategoryService,
) *BudgetService {
	return &BudgetService{
		db:              db,
		categoryService: categoryService,
	}
}

func (s *BudgetService) AddBudget(ctx context.Context, budget *BudgetEntity) (*BudgetEntity, error) {
	userId, err := context_utils.GetUserId(ctx)
	if err != nil {
		return nil, err
	}

	tx, err := s.db.Begin(ctx)
	if err != nil {
		return nil, err
	}
	defer tx.Rollback(ctx)

	query := `INSERT INTO budgets (name, start_day, monthly_limit) VALUES (@name, @start_day, @monthly_limit) RETURNING id`
	args := pgx.NamedArgs{
		"name":          budget.Name,
		"start_day":     budget.StartDay,
		"monthly_limit": budget.MonthlyLimit,
	}
	if err := s.db.QueryRow(ctx, query, args).Scan(&budget.Id); err != nil {
		return nil, fmt.Errorf("unable to insert row: %w", err)
	}

	_, err = s.AddUserToBudget(ctx, budget.Id, userId)
	if err != nil {
		return nil, err
	}

	defaultCategories := []*category.CategoryEntity{
		{
			Name:     "Транспорт",
			BudgetId: budget.Id,
		},
		{
			Name:     "Продукты",
			BudgetId: budget.Id,
		},
	}
	for _, category := range defaultCategories {
		_, err = s.categoryService.AddCategory(ctx, category)
		if err != nil {
			return nil, err
		}
	}

	if err = tx.Commit(ctx); err != nil {
		return nil, err
	}
	return budget, nil
}

func (s *BudgetService) GetBudgets(ctx context.Context) ([]*BudgetEntity, error) {
	userId, err := context_utils.GetUserId(ctx)
	if err != nil {
		return nil, err
	}

	query := `SELECT b.id, b.name, b.start_day, b.monthly_limit FROM budgets b JOIN users_budgets ub ON ub.budget_id = b.id WHERE ub.user_id = @user_id`
	args := pgx.NamedArgs{
		"user_id": userId,
	}
	rows, err := s.db.Query(ctx, query, args)
	if err != nil {
		return nil, err
	}
	budgets := []*BudgetEntity{}
	defer rows.Close()
	for rows.Next() {
		budget := &BudgetEntity{}
		err = rows.Scan(&budget.Id, &budget.Name, &budget.StartDay, &budget.MonthlyLimit)
		if err != nil {
			return nil, err
		}
		budget.Categories, err = s.GetBudgetCategories(ctx, budget.Id)
		budgets = append(budgets, budget)
	}

	return budgets, nil
}

func (s *BudgetService) AddUserToBudget(
	ctx context.Context,
	budgetId int,
	userId int,
) (int, error) {
	query := `INSERT INTO users_budgets (user_id, budget_id) VALUES (@user_id, @budget_id) RETURNING id`
	args := pgx.NamedArgs{
		"user_id":   userId,
		"budget_id": budgetId,
	}
	id := 0
	if err := s.db.QueryRow(ctx, query, args).Scan(&id); err != nil {
		return 0, fmt.Errorf("unable to insert row: %w", err)
	}
	return id, nil
}

func (s *BudgetService) GetBudgetCategories(
	ctx context.Context,
	budgetId int,
) ([]*category.CategoryEntity, error) {
	query := `SELECT id, name, favorite, monthly_limit FROM categories WHERE budget_id = @budget_id`
	args := pgx.NamedArgs{
		"budget_id": budgetId,
	}
	rows, err := s.db.Query(ctx, query, args)
	if err != nil {
		return nil, err
	}
	categories := []*category.CategoryEntity{}
	defer rows.Close()
	for rows.Next() {
		category := &category.CategoryEntity{}
		err = rows.Scan(&category.Id, &category.Name, &category.Favorite, &category.MonthlyLimit)
		if err != nil {
			return nil, err
		}
		categories = append(categories, category)
	}

	return categories, nil
}
